#include <avr/io.h>

#include "lights.h"

static unsigned char led_counters[N_LEDS];
static pattern_t *led_patterns[N_LEDS];

pattern_t off_pattern[] = {
	{ 0, PATTERN_FOREVER },
};

pattern_t on_pattern[] = {
	{ 1, PATTERN_FOREVER },
};

pattern_t blink_pattern[] = {
	{ 1, 0x4 },
	{ 0, 0x8 },
	PATTERN_END
};

pattern_t mode1_pattern[] = {
	{ 4, 0x1 },
	{ 0, 0x1 },
	{ 4, 0x1 },
	{ 0, 0x8 },
	{ 1, 0x1 },
	{ 0, 0x1 },
	{ 1, 0x1 },
	{ 0, 0x8 },
	PATTERN_END
};

pattern_t boot_pattern[] = {
	{ 1, 0x6 },
	{ 0, 0x6 },
	{ 1, 0x3 },
	{ 0, 0x3 },
	{ 1, 0x2 },
	{ 0, 0x2 },
	{ 1, 0x1 },
	{ 0, 0x1 },
	{ 1, 0x1 },
	{ 0, 0x1 },
	{ 1, 0x1 },
	{ 0, 0x1 },
	{ 1, 0x1 },
	{ 0, 0x1 },
	{ 1, 0x10 },
	{ 0, 0x10 },
	PATTERN_END
};

pattern_t pattern_num[] = {
	{ 0, 0x5 },
	{ 1, 0x1 }, /* 10 */
	{ 0, 0x5 },
	{ 1, 0x1 }, /*  9 */
	{ 0, 0x5 },
	{ 1, 0x1 }, /*  8 */
	{ 0, 0x5 },
	{ 1, 0x1 }, /*  7 */
	{ 0, 0x5 },
	{ 1, 0x1 }, /*  6 */
	{ 0, 0x5 },
	{ 1, 0x1 }, /*  5 */
	{ 0, 0x5 },
	{ 1, 0x1 }, /*  4 */
	{ 0, 0x5 },
	{ 1, 0x1 }, /*  3 */
	{ 0, 0x5 },
	{ 1, 0x1 }, /*  2 */
	{ 0, 0x5 },
	{ 1, 0x1 }, /*  1 */
	{ 0, 0x1E },
	PATTERN_END
};

static unsigned char test_running;

static void led_set_mode(unsigned char n, unsigned char mode)
{
	if (n < N_PWMLEDS) {
		pwmled_set_mode(n, mode);
	} else if (n < N_LEDS) {
		gpio_set(n - N_PWMLEDS, mode);
	}
}

void led_set_pattern(unsigned char n, pattern_t *pattern)
{
	if (!pattern)
		pattern = off_pattern;

	led_patterns[n] = pattern;
	led_counters[n] = pattern->duration;
	led_set_mode(n, pattern->mode);
}

void pattern_init()
{
	unsigned char i;

	for (i = 0; i < N_LEDS; i++)
		led_set_pattern(i, off_pattern);

	led_set_pattern(N_PWMLEDS+1, boot_pattern);
	test_running = 0;
}

pattern_t *number_pattern(unsigned char num)
{
	if (num >= 9)
		num = 9;

	return pattern_num + sizeof(pattern_num)/sizeof(pattern_t)
		- 2 - 2*num;
}

static inline pattern_t *pattern_select(unsigned char n)
{
	if (n < N_PWMLEDS && !pwmled_enabled(n))
		return off_pattern; // Don't mess with non-enabled LEDs
	else if (n == 2)
		return mode1_pattern;
	else if (n == 3)
		return status_pattern_select(0);
	else if (n == 4)
		return status_pattern_select(1);
	return off_pattern;
}

void patterns_next_tick()
{
	unsigned char i;

	for (i = 0; i < N_LEDS; i++) {
		if (led_counters[i] == 0) {
			pattern_t *p = led_patterns[i];
			p++;
			if (p->duration == 0) { // END
				p = pattern_select(i);
			}
			led_set_pattern(i, p);
		}

		if (led_counters[i] < PATTERN_FOREVER)
			led_counters[i]--;
	}
}

void led_set_status(unsigned char status)
{
	led_set_pattern(N_PWMLEDS+1, number_pattern(status));
}
